import React, { useState, useEffect, useRef, useCallback } from "react";
import { useTheme } from "../hooks/useTheme";
import { tokenize } from "../utils/tokenizer";
import type { HighlighterProps, Token } from "../types";
import { 
  Copy, 
  Check, 
  ChevronDown, 
  ChevronUp, 
  Code,
  Search,
  Sun,
  Moon,
  Share2,
  Download
} from "lucide-react";

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = true,
  wrapLines = false,
  maxHeight,
  isDarkMode,
  expandable = true,
  className = "",
  style = {},
  customBackgroundColor,
  showCopyButton = true,
  fileName = "",
  showSearch = true,
  showShare = true,
  showThemeToggle = true,
  showDownload = true,
}) => {
  const { theme } = useTheme();
  const [tokens, setTokens] = useState<Token[]>([]);
  const [copied, setCopied] = useState(false);
  const [isExpanded, setIsExpanded] = useState(!expandable);
  const [searchTerm, setSearchTerm] = useState("");
  const [searchResults, setSearchResults] = useState<number[]>([]);
  const [currentSearchIndex, setCurrentSearchIndex] = useState(-1);
  const codeRef = useRef<HTMLPreElement>(null);

  useEffect(() => {
    setTokens(tokenize(code, language));
  }, [code, language]);

  const copyToClipboard = async () => {
    try {
      await navigator.clipboard.writeText(code);
      setCopied(true);
      setTimeout(() => setCopied(false), 2000);
    } catch (err) {
      console.error("Failed to copy:", err);
    }
  };

  const handleSearch = useCallback(() => {
    if (!searchTerm) {
      setSearchResults([]);
      setCurrentSearchIndex(-1);
      return;
    }

    const results = code.split("\n").reduce((acc: number[], line, index) => {
      if (line.toLowerCase().includes(searchTerm.toLowerCase())) {
        acc.push(index);
      }
      return acc;
    }, []);

    setSearchResults(results);
    setCurrentSearchIndex(results.length > 0 ? 0 : -1);

    if (results.length > 0 && codeRef.current) {
      const lineElement = codeRef.current.querySelector(`[data-line="${results[0]}"]`);
      lineElement?.scrollIntoView({ behavior: "smooth", block: "center" });
    }
  }, [searchTerm, code]);

  useEffect(() => {
    handleSearch();
  }, [handleSearch]);

  const navigateSearch = (direction: "next" | "prev") => {
    if (searchResults.length === 0) return;

    let newIndex;
    if (direction === "next") {
      newIndex = (currentSearchIndex + 1) % searchResults.length;
    } else {
      newIndex = (currentSearchIndex - 1 + searchResults.length) % searchResults.length;
    }

    setCurrentSearchIndex(newIndex);

    if (codeRef.current) {
      const lineElement = codeRef.current.querySelector(`[data-line="${searchResults[newIndex]}"]`);
      lineElement?.scrollIntoView({ behavior: "smooth", block: "center" });
    }
  };

  const shareCode = async () => {
    try {
      await navigator.share({
        title: `Code Snippet - ${language}`,
        text: code,
        url: window.location.href,
      });
    } catch (err) {
      console.error("Failed to share:", err);
    }
  };

  const downloadCode = () => {
    const element = document.createElement("a");
    const file = new Blob([code], { type: "text/plain" });
    element.href = URL.createObjectURL(file);
    element.download = fileName || `code.${language}`;
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
  };

  return (
    <div className="rounded-xl border border-gray-200 dark:border-gray-800 bg-white dark:bg-gray-900 shadow-lg overflow-hidden w-full">
      <div className="flex items-center justify-between px-4 py-3 bg-gray-50 dark:bg-gray-800">
        <div className="flex items-center space-x-2">
          <Code className="w-5 h-5 text-gray-600 dark:text-gray-300" />
          <span className="text-sm font-medium text-gray-700 dark:text-gray-200">
            {language.toUpperCase()} {fileName}
          </span>
        </div>
        <div className="flex items-center space-x-2">
          {showThemeToggle && (
            <button
              onClick={() => document.documentElement.classList.toggle("dark")}
              className="p-1 rounded hover:bg-gray-200 dark:hover:bg-gray-700 transition-all duration-200"
            >
              <Sun className="w-5 h-5 block dark:hidden" />
              <Moon className="w-5 h-5 hidden dark:block" />
            </button>
          )}
          {showShare && (
            <button
              onClick={shareCode}
              className="p-1 rounded hover:bg-gray-200 dark:hover:bg-gray-700 transition-all duration-200"
            >
              <Share2 className="w-5 h-5" />
            </button>
          )}
          {showDownload && (
            <button
              onClick={downloadCode}
              className="p-1 rounded hover:bg-gray-200 dark:hover:bg-gray-700 transition-all duration-200"
            >
              <Download className="w-5 h-5" />
            </button>
          )}
          {showCopyButton && (
            <button
              onClick={copyToClipboard}
              className="p-1 rounded hover:bg-gray-200 dark:hover:bg-gray-700 transition-all duration-200"
            >
              {copied ? <Check className="w-5 h-5" /> : <Copy className="w-5 h-5" />}
            </button>
          )}
        </div>
      </div>

      {showSearch && (
        <div className="p-4 bg-gray-100 dark:bg-gray-800 border-b border-gray-200 dark:border-gray-700">
          <div className="flex items-center space-x-2">
            <Search className="w-5 h-5 text-gray-400" />
            <input
              type="text"
              value={searchTerm}
              onChange={(e) => setSearchTerm(e.target.value)}
              placeholder="Search in code..."
              className="flex-1 px-3 py-1 text-sm bg-white dark:bg-gray-700 border border-gray-300 dark:border-gray-600 rounded-md"
            />
            {searchTerm && (
              <div className="flex items-center space-x-2">
                <button
                  onClick={() => navigateSearch("prev")}
                  className="px-2 py-1 text-sm bg-gray-200 dark:bg-gray-700 rounded transition-colors duration-200"
                >
                  ↑
                </button>
                <button
                  onClick={() => navigateSearch("next")}
                  className="px-2 py-1 text-sm bg-gray-200 dark:bg-gray-700 rounded transition-colors duration-200"
                >
                  ↓
                </button>
                <span className="text-sm text-gray-500">
                  {searchResults.length > 0 ? `${currentSearchIndex + 1}/${searchResults.length}` : "No results"}
                </span>
              </div>
            )}
          </div>
        </div>
      )}

      <div className="relative" style={{ backgroundColor: customBackgroundColor || theme.background }}>
        <pre
          ref={codeRef}
          className={`p-4 text-sm overflow-x-auto ${!isExpanded ? "max-h-[400px]" : ""}`}
          style={{
            ...style,
            whiteSpace: wrapLines ? "pre-wrap" : "pre",
          }}
        >
          <code>
            {code.split("\n").map((line, lineIndex) => (
              <div
                key={lineIndex}
                className={`flex ${
                  searchResults.includes(lineIndex)
                    ? "bg-yellow-100 dark:bg-yellow-900 bg-opacity-50"
                    : ""
                }`}
                data-line={lineIndex}
              >
                {showLineNumbers && (
                  <span
                    className="select-none mr-4 text-gray-400 dark:text-gray-600 text-right w-8"
                    style={{ color: theme.lineNumber }}
                  >
                    {lineIndex + 1}
                  </span>
                )}
                <span>
                  {tokenize(line, language).map((token, tokenIndex) => (
                    <span
                      key={tokenIndex}
                      style={{ color: theme[token.type as keyof typeof theme] || theme.text }}
                    >
                      {token.content}
                    </span>
                  ))}
                </span>
              </div>
            ))}
          </code>
        </pre>

        {expandable && !isExpanded && (
          <div className="absolute bottom-0 left-0 right-0 h-24 bg-gradient-to-t from-white dark:from-gray-900 to-transparent" />
        )}
      </div>

      {expandable && (
        <button
          onClick={() => setIsExpanded(!isExpanded)}
          className="w-full p-2 flex items-center justify-center space-x-2 bg-gray-50 dark:bg-gray-800 hover:bg-gray-100 dark:hover:bg-gray-700 transition-all duration-200"
        >
          <div className="flex items-center space-x-2">
            {isExpanded ? (
              <>
                <ChevronUp className="w-5 h-5" />
                <span>Show less</span>
              </>
            ) : (
              <>
                <ChevronDown className="w-5 h-5" />
                <span>Show more</span>
              </>
            )}
          </div>
        </button>
      )}
    </div>
  );
};

export default SyntaxHighlighter;
